@@ -12,10 +12,29 @@ import scipy.sparse.linalg
 
 
 class Index():
+    """Convenience class used as a an array, to be used with index_update
+
+    Parameters
+    ----------
+    indices : indices for indexing
+
+    Examples
+    --------
+    Usage: index[indices], e.g. ::
+
+        index[1:3, 4:5, :None]
+    
+    See also
+    --------
+    index_update : updating the values of a tensor for specified indices
+    """
     __slots__ = ()
 
     def __getitem__(self, indices):
         return indices
+    @property
+    def __name__(self):
+        return 'Index'
 
 
 class Backend(object):
@@ -49,9 +68,61 @@ class Backend(object):
         raise NotImplementedError
 
     @property
-    def SVD_FUNS(self):
+    def complex128(self):
         raise NotImplementedError
 
+    @property
+    def complex64(self):
+        raise NotImplementedError
+
+    @staticmethod
+    def check_random_state(seed):
+        """Returns a valid RandomState
+
+        Parameters
+        ----------
+        seed : None or instance of int or np.random.RandomState(), default is None
+        if seed is None NumPy's global seed is used.
+
+        Returns
+        -------
+        Valid instance np.random.RandomState
+
+        Notes
+        -----
+        Inspired by the scikit-learn eponymous function
+        """
+        if seed is None:
+            return np.random.mtrand._rand
+
+        elif isinstance(seed, int):
+            return np.random.RandomState(seed)
+
+        elif isinstance(seed, np.random.RandomState):
+            return seed
+
+        raise ValueError('Seed should be None, int or np.random.RandomState')
+
+    def randn(self, shape, seed=None, **context):
+        """Returns a random tensor with samples from the “standard normal” distribution.
+
+        Parameters
+        ----------
+        shape: Iterable[int]
+            shape of the random tensor
+        seed: None or instance of int or np.random.RandomState(), default is None
+        if seed is None NumPy's global seed is used
+        context: context of tensor
+
+        Returns
+        -------
+        random_tensor: tl.tensor
+        """
+        rng = self.check_random_state(seed)
+        random_tensor = rng.randn(*shape)
+        random_tensor = self.tensor(random_tensor, **context)
+        return random_tensor
+
     @staticmethod
     def context(tensor):
         """Returns the context of a tensor
@@ -463,8 +534,7 @@ class Backend(object):
         """
         raise NotImplementedError
 
-    @staticmethod
-    def norm(tensor, order=2, axis=None):
+    def norm(self, tensor, order=2, axis=None):
         """Computes the l-`order` norm of a tensor.
 
         Parameters
@@ -478,7 +548,18 @@ class Backend(object):
         float or tensor
             If `axis` is provided returns a tensor.
         """
-        raise NotImplementedError
+        # handle difference in default axis notation
+        if axis == ():
+            axis = None
+
+        if order == 'inf':
+            return self.max(self.abs(tensor), axis=axis)
+        if order == 1:
+            return self.sum(self.abs(tensor), axis=axis)
+        elif order == 2:
+            return self.sqrt(self.sum(self.abs(tensor)**2, axis=axis))
+        else:
+            return self.sum(self.abs(tensor)**order, axis=axis)**(1 / order)
 
     @staticmethod
     def dot(a, b):
@@ -495,6 +576,53 @@ class Backend(object):
         """
         raise NotImplementedError
 
+    @staticmethod
+    def tensordot(a, b, axes=2):
+        """
+        Compute tensor dot product along specified axes.
+        Given two tensors, `a` and `b`, and an array_like object containing
+        two array_like objects, ``(a_axes, b_axes)``, sum the products of
+        `a`'s and `b`'s elements (components) over the axes specified by
+        ``a_axes`` and ``b_axes``. The third argument can be a single non-negative
+        integer_like scalar, ``N``; if it is such, then the last ``N`` dimensions
+        of `a` and the first ``N`` dimensions of `b` are summed over.
+
+        Parameters
+        ----------
+        a, b : array_like
+            Tensors to "dot".
+        axes : int or (2,) array_like
+            * integer_like
+            If an int N, sum over the last N axes of `a` and the first N axes
+            of `b` in order. The sizes of the corresponding axes must match.
+            * (2,) array_like
+            Or, a list of axes to be summed over, first sequence applying to `a`,
+            second to `b`. Both elements array_like must be of the same length.
+
+        Returns
+        -------
+        output : ndarray
+            The tensor dot product of the input.
+
+        Notes
+        -----
+        Three common use cases are:
+            * ``axes = 0`` : tensor product :math:`a\\otimes b`
+            * ``axes = 1`` : tensor dot product :math:`a\\cdot b`
+            * ``axes = 2`` : (default) tensor double contraction :math:`a:b`
+
+        When `axes` is integer_like, the sequence for evaluation will be: first
+        the -Nth axis in `a` and 0th axis in `b`, and the -1th axis in `a` and
+        Nth axis in `b` last.
+        When there is more than one axis to sum over - and they are not the last
+        (first) axes of `a` (`b`) - the argument `axes` should consist of
+        two sequences of the same length, with the first axis to sum over given
+        first in both sequences, the second axis second, and so forth.
+        The shape of the result consists of the non-contracted axes of the
+        first tensor, followed by the non-contracted axes of the second.
+        """
+        raise NotImplementedError
+
     @staticmethod
     def solve(a, b):
         """Solve a linear matrix equation, or system of linear scalar equations.
@@ -542,16 +670,35 @@ class Backend(object):
         raise NotImplementedError
 
     def eps(self, dtype):
+        """Returns the machine epsilon for a given floating point dtype
+
+        Parameters
+        ----------
+        dtype : tensorly.dtype
+            the dtype for which to get the machine epsilon
+        
+        Returns
+        -------
+        eps : machine epsilon for `dtype`
+        """
         return self.finfo(dtype).eps
 
     def finfo(self, dtype):
+        """Machine limits for floating point types.
+
+        Parameters
+        ----------
+        dtype: float, dtype or instance
+                Kind of floating point data-type about which to get information.
+        """
         return np.finfo(self.to_numpy(self.tensor([], dtype=dtype)).dtype)
 
     @staticmethod
     def conj(x, *args, **kwargs):
         """Return the complex conjugate, element-wise.
 
-            The complex conjugate of a complex number is obtained by changing the sign of its imaginary part.
+            The complex conjugate of a complex number is obtained by 
+            changing the sign of its imaginary part.
         """
         raise NotImplementedError
 
@@ -576,6 +723,7 @@ class Backend(object):
         """
         raise NotImplementedError
 
+    @staticmethod
     def einsum(subscripts, *operands):
         """Evaluates the Einstein summation convention on the operands.
 
@@ -591,6 +739,10 @@ class Backend(object):
         -------
         output : ndarray
             The calculation based on the Einstein summation convention
+
+        Notes
+        -----
+        This is only available for certain backends.
         """
         raise NotImplementedError
 
@@ -610,6 +762,7 @@ class Backend(object):
         -------
         tensor
         """
+
         axes = list(range(self.ndim(tensor)))
         if source < 0: source = axes[source]
         if destination < 0: destination = axes[destination]
@@ -624,7 +777,7 @@ class Backend(object):
             raise ValueError('Destination should verify 0 <= destination < tensor.ndim'
                              'Got %d' % destination)
         return self.transpose(tensor, axes)
-
+    
     def kron(self, a, b):
         """Kronecker product of two tensors.
 
@@ -728,48 +881,26 @@ class Backend(object):
 
         ctx = self.context(matrix)
         is_numpy = isinstance(matrix, np.ndarray)
-
         if not is_numpy:
-            matrix = self.to_numpy(matrix)
+            warnings.warn('In partial_svd: converting to NumPy.'
+                          ' Check SVD_FUNS for available alternatives if you want to avoid this.')
 
         # Choose what to do depending on the params
-        dim_1, dim_2 = matrix.shape
-        if dim_1 <= dim_2:
-            min_dim = dim_1
-            max_dim = dim_2
-        else:
-            min_dim = dim_2
-            max_dim = dim_1
+        dim_1, dim_2 = self.shape(matrix)
+        min_dim, max_dim = (dim_1, dim_2) if dim_1 < dim_2 else (dim_2, dim_1)
+        matrix = self.to_numpy(matrix)
 
-        if n_eigenvecs is None:
-            # Default on standard SVD
-            U, S, V = scipy.linalg.svd(matrix, full_matrices=True)
-            U, S, V = U[:, :n_eigenvecs], S[:n_eigenvecs], V[:n_eigenvecs, :]
-        elif min_dim <= n_eigenvecs:
-            if max_dim < n_eigenvecs:
-                warnings.warn(('Trying to compute SVD with n_eigenvecs={0}, which '
-                               'is larger than max(matrix.shape)={1}. Setting '
-                               'n_eigenvecs to {1}').format(n_eigenvecs, max_dim))
-                n_eigenvecs = max_dim
-            if n_eigenvecs > min_dim:
-                full_matrices=True
-            else:
-                full_matrices=False
+        if (n_eigenvecs is None) or (n_eigenvecs >= min_dim):
+            # Just perform trucated SVD
+            full_matrices = (n_eigenvecs is None) or (n_eigenvecs > min_dim)
+            # If n_eigenvecs == min_dim, we don't want full_matrices=True, it's super slow
             U, S, V = scipy.linalg.svd(matrix, full_matrices=full_matrices)
             U, S, V = U[:, :n_eigenvecs], S[:n_eigenvecs], V[:n_eigenvecs, :]
         else:
             # We can perform a partial SVD
-            # construct np.random.RandomState for sampling a starting vector
-            if random_state is None:
-                # if random_state is not specified, do not initialize a starting vector
-                v0 = None
-            elif isinstance(random_state, int):
-                rns = np.random.RandomState(random_state)
-                # initilize with [-1, 1] as in ARPACK
-                v0 = rns.uniform(-1, 1, min_dim)
-            elif isinstance(random_state, np.random.RandomState):
-                # initilize with [-1, 1] as in ARPACK
-                v0 = random_state.uniform(-1, 1, min_dim)
+            rng = self.check_random_state(random_state)
+            # initilize with [-1, 1] as in ARPACK
+            v0 = rng.uniform(-1, 1, min_dim)
 
             # First choose whether to use X * X.T or X.T *X
             if dim_1 < dim_2:
@@ -783,7 +914,7 @@ class Backend(object):
                     np.dot(matrix.T.conj(), matrix), k=n_eigenvecs, which='LM', v0=v0
                 )
                 S = np.where(np.abs(S) <= np.finfo(S.dtype).eps, 0, np.sqrt(S))
-                U = np.dot(matrix, V) *  np.where(np.abs(S) <= np.finfo(S.dtype).eps, 0, 1/S)[None, :]
+                U = np.dot(matrix, V) * np.where(np.abs(S) <= np.finfo(S.dtype).eps, 0, 1/S)[None, :]
 
             # WARNING: here, V is still the transpose of what it should be
             U, S, V = U[:, ::-1], S[::-1], V[:, ::-1]
@@ -793,10 +924,57 @@ class Backend(object):
             U = self.tensor(U, **ctx)
             S = self.tensor(S, **ctx)
             V = self.tensor(V, **ctx)
+
         return U, S, V
 
     def truncated_svd(self, matrix, n_eigenvecs=None, **kwargs):
-        """Computes a truncated SVD on `matrix` using pytorch's SVD
+        """Computes a truncated SVD on `matrix` using the backends's standard SVD
+
+        Parameters
+        ----------
+        matrix : 2D-array
+        n_eigenvecs : int, optional, default is None
+            if specified, number of eigen[vectors-values] to return
+
+        Returns
+        -------
+        U : 2D-array
+            of shape (matrix.shape[0], n_eigenvecs)
+            contains the right singular vectors
+        S : 1D-array
+            of shape (n_eigenvecs, )
+            contains the singular values of `matrix`
+        V : 2D-array
+            of shape (n_eigenvecs, matrix.shape[1])
+            contains the left singular vectors
+        """
+        # Check that matrix is... a matrix!
+        if self.ndim(matrix) != 2:
+            raise ValueError('matrix be a matrix. matrix.ndim is %d != 2'
+                             % self.ndim(matrix))
+
+        dim_1, dim_2 = self.shape(matrix)
+        min_dim, max_dim = min(dim_1, dim_2), max(dim_1, dim_2)
+
+        if n_eigenvecs is None:
+            n_eigenvecs = max_dim
+
+        if n_eigenvecs > max_dim:
+            warnings.warn('Trying to compute SVD with n_eigenvecs={0}, which '
+                          'is larger than max(matrix.shape)={1}. Setting '
+                          'n_eigenvecs to {1}'.format(n_eigenvecs, max_dim))
+            n_eigenvecs = max_dim
+
+        full_matrices = n_eigenvecs > min_dim
+
+        U, S, V = self.svd(matrix, full_matrices=full_matrices)
+        U, S, V = U[:, :n_eigenvecs], S[:n_eigenvecs], V[:n_eigenvecs, :]
+        return U, S, V
+
+    def symeig_svd(self, matrix, n_eigenvecs=None, **kwargs):
+        """Computes a truncated SVD on `matrix` using symeig
+
+            Uses symeig on matrix.T.dot(matrix) or its transpose
 
         Parameters
         ----------
@@ -818,22 +996,149 @@ class Backend(object):
             of shape (n_eigenvecs, matrix.shape[1])
             contains the left singular vectors
         """
-        dim_1, dim_2 = matrix.shape
-        if dim_1 <= dim_2:
-            min_dim = dim_1
-        else:
-            min_dim = dim_2
+        # Check that matrix is... a matrix!
+        if self.ndim(matrix) != 2:
+            raise ValueError('matrix be a matrix. matrix.ndim is %d != 2'
+                             % self.ndim(matrix))
 
-        if n_eigenvecs is None or n_eigenvecs > min_dim:
-            full_matrices = True
+        dim_1, dim_2 = self.shape(matrix)
+        min_dim, max_dim = min(dim_1, dim_2), max(dim_1, dim_2)
+
+        if n_eigenvecs is None:
+            n_eigenvecs = max_dim
+
+        if n_eigenvecs > max_dim:
+            warnings.warn('Trying to compute SVD with n_eigenvecs={0}, which '
+                          'is larger than max(matrix.shape)={1}. Setting '
+                          'n_eigenvecs to {1}'.format(n_eigenvecs, max_dim))
+            n_eigenvecs = max_dim
+
+        if dim_1 > dim_2:
+            S, U = self.eigh(self.dot(matrix, self.transpose(matrix)))
+            S = self.sqrt(self.clip(S, self.eps(S.dtype)))
+            V = self.dot(self.transpose(matrix), U / self.reshape(S, (1, -1)))
         else:
-            full_matrices = False
+            S, V = self.eigh(self.dot(self.transpose(matrix), matrix))
+            S = self.sqrt(self.clip(S, self.eps(S.dtype)))
+            U = self.dot(matrix, V) / self.reshape(S, (1, -1))
+
+        U, S, V = self.flip(U, axis=1), self.flip(S), self.flip(self.transpose(V), axis=0)
+        return U[:, :min(dim_1, n_eigenvecs)], S[:min(dim_1, dim_2, n_eigenvecs)], V[:min(dim_2, n_eigenvecs), :]
+
+    def randomized_svd(self, matrix, n_eigenvecs=None, n_oversamples=5, n_iter=2, random_state=None, **kwargs):
+        """Computes a truncated randomized SVD.
+
+        If `n_eigenvecs` is specified, sparse eigendecomposition is used on
+        either matrix.dot(matrix.T) or matrix.T.dot(matrix).
+
+        Parameters
+        ----------
+        matrix : tensor
+            A 2D tensor.
+        n_eigenvecs : int, optional, default is None
+            If specified, number of eigen[vectors-values] to return.
+        n_oversamples: int, optional, default = 5
+            rank overestimation value for finiding subspace with better allignment
+        n_iter: int, optional, default = 2
+            number of power iterations for the `randomized_range_finder` subroutine
+        random_state: {None, int, np.random.RandomState}
+        **kwargs : optional
+            kwargs are used to absorb the difference of parameters among the other SVD functions
+
+        Returns
+        -------
+        U : 2-D tensor, shape (matrix.shape[0], n_eigenvecs)
+            Contains the right singular vectors
+        S : 1-D tensor, shape (n_eigenvecs, )
+            Contains the singular values of `matrix`
+        V : 2-D tensor, shape (n_eigenvecs, matrix.shape[1])
+            Contains the left singular vectors
+
+        Notes
+        -----
+        This function is implemented based on Algorith 5.1 in `Finding structure with randomness:
+        Probabilistic algorithms for constructing approximate matrix decompositions`
+        - Halko et al (2009)
+        """
+        # Check that matrix is... a matrix!
+        if self.ndim(matrix) != 2:
+            raise ValueError('matrix be a matrix. matrix.ndim is %d != 2'
+                             % self.ndim(matrix))
+
+        dim_1, dim_2 = self.shape(matrix)
+        min_dim, max_dim = min(dim_1, dim_2), max(dim_1, dim_2)
+
+        if n_eigenvecs is None:
+            n_eigenvecs = max_dim
+
+        if n_eigenvecs > max_dim:
+            warnings.warn('Trying to compute SVD with n_eigenvecs={0}, which '
+                          'is larger than max(matrix.shape)={1}. Setting '
+                          'n_eigenvecs to {1}'.format(n_eigenvecs, max_dim))
+            n_eigenvecs = max_dim
+
+        n_dims = min(n_eigenvecs + n_oversamples, max_dim)
+
+        if dim_1 > dim_2 and n_eigenvecs > min(min_dim, n_dims) or \
+           dim_1 < dim_2 and n_eigenvecs < min(min_dim, n_dims):
+            # transpose matrix to keep the reduced matrix shape minimal
+            matrix_T = self.transpose(matrix)
+            Q = self.randomized_range_finder(matrix_T, n_dims=n_dims, n_iter=n_iter, random_state=random_state)
+            Q_H = self.conj(self.transpose(Q))
+            matrix_reduced = self.transpose(self.dot(Q_H, matrix_T))
+            U, S, V = self.truncated_svd(matrix_reduced, n_eigenvecs=n_eigenvecs)
+            V = self.dot(V, self.transpose(Q))
+        else:
+            Q = self.randomized_range_finder(matrix, n_dims=n_dims, n_iter=n_iter, random_state=random_state)
+            Q_H = self.conj(self.transpose(Q))
+            matrix_reduced = self.dot(Q_H, matrix)
+            U, S, V = self.truncated_svd(matrix_reduced, n_eigenvecs=n_eigenvecs)
+            U = self.dot(Q, U)
 
-        U, S, V = self.svd(matrix)
-        U, S, V = U[:, :n_eigenvecs], S[:n_eigenvecs], V[:n_eigenvecs, :]
         return U, S, V
 
+    def randomized_range_finder(self, A, n_dims, n_iter=2, random_state=None):
+        """Computes an orthonormal matrix (Q) whose range approximates the range of A,  i.e., Q Q^H A ≈ A
+
+        Parameters
+        ----------
+        A : 2D-array
+        n_dims : int, dimension of the returned subspace
+        n_iter : int, number of power iterations to conduct (default = 2)
+        random_state: {None, int, np.random.RandomState}
+
+        Returns
+        -------
+        Q : 2D-array
+            of shape (A.shape[0], min(n_dims, A.shape[0], A.shape[1]))
+
+        Notes
+        -----
+        This function is implemented based on Algorith 4.4 in `Finding structure with randomness:
+        Probabilistic algorithms for constructing approximate matrix decompositions`
+        - Halko et al (2009)
+        """
+        rng = self.check_random_state(random_state)
+        dim_1, dim_2 = self.shape(A)
+        Q = self.tensor(rng.normal(size=(dim_2, n_dims)), **self.context(A))
+        Q, _ = self.qr(self.dot(A, Q))
+
+        # Perform power iterations when spectrum decays slowly
+        A_H = self.conj(self.transpose(A))
+        for i in range(n_iter):
+            Q, _ = self.qr(self.dot(A_H, Q))
+            Q, _ = self.qr(self.dot(A, Q))
+
+        return Q
+
     index = Index()
+
+    @property
+    def SVD_FUNS(self):
+        return {'numpy_svd': self.partial_svd,
+                'truncated_svd': self.truncated_svd,
+                'symeig_svd': self.symeig_svd,
+                'randomized_svd': self.randomized_svd}
     
     @staticmethod
     def index_update(tensor, indices, values):
@@ -874,6 +1179,16 @@ class Backend(object):
         >>> tl.index_update(tensor, tl.index[:, 1], 0)
         array([[1, 0, 3],
                [4, 0, 6]])
+
+        See also
+        --------
+        index
         """
         tensor[indices] = values
-        return tensor
\ No newline at end of file
+        return tensor
+
+    @staticmethod
+    def log2(x):
+        """Return the base 2 logarithm of x.
+        """
+        raise NotImplementedError
