@@ -3,7 +3,7 @@ import warnings
 
 import tensorly as tl
 from ._base_decomposition import DecompositionMixin
-from ..random import check_random_state, random_cp
+from ..random import random_cp
 from ..base import unfold
 from ..cp_tensor import (cp_to_tensor, CPTensor,
                          unfolding_dot_khatri_rao, cp_norm,
@@ -16,20 +16,20 @@ from ..cp_tensor import (cp_to_tensor, CPTensor,
 
 # License: BSD 3 clause
 
-def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None, 
-                       non_negative=False, normalize_factors=False):
+def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None, normalize_factors=False):
     r"""Initialize factors used in `parafac`.
 
     The type of initialization is set using `init`. If `init == 'random'` then
-    initialize factor matrices using `random_state`. If `init == 'svd'` then
+    initialize factor matrices with uniform distribution using `random_state`. If `init == 'svd'` then
     initialize the `m`th factor matrix using the `rank` left singular vectors
-    of the `m`th unfolding of the input tensor.
+    of the `m`th unfolding of the input tensor. If init is a previously initialized `cp tensor`, all
+    the weights are pulled in the last factor and then the weights are set to "1" for the output tensor.
 
     Parameters
     ----------
     tensor : ndarray
     rank : int
-    init : {'svd', 'random'}, optional
+    init : {'svd', 'random', cptensor}, optional
     svd : str, default is 'numpy_svd'
         function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
     non_negative : bool, default is False
@@ -41,19 +41,17 @@ def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None,
         An initial cp tensor.
 
     """
-    rng = check_random_state(random_state)
+    rng = tl.check_random_state(random_state)
 
     if init == 'random':
-        # factors = [tl.tensor(rng.random_sample((tensor.shape[i], rank)), **tl.context(tensor)) for i in range(tl.ndim(tensor))]
-        # kt = CPTensor((None, factors))
-        return random_cp(tl.shape(tensor), rank, normalise_factors=False, **tl.context(tensor))
+        kt = random_cp(tl.shape(tensor), rank, normalise_factors=False, **tl.context(tensor))
 
     elif init == 'svd':
         try:
             svd_fun = tl.SVD_FUNS[svd]
         except KeyError:
             message = 'Got svd={}. However, for the current backend ({}), the possible choices are {}'.format(
-                    svd, tl.get_backend(), tl.SVD_FUNS)
+                      svd, tl.get_backend(), tl.SVD_FUNS)
             raise ValueError(message)
 
         factors = []
@@ -67,9 +65,6 @@ def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None,
 
             if tensor.shape[mode] < rank:
                 # TODO: this is a hack but it seems to do the job for now
-                # factor = tl.tensor(np.zeros((U.shape[0], rank)), **tl.context(tensor))
-                # factor[:, tensor.shape[mode]:] = tl.tensor(rng.random_sample((U.shape[0], rank - tl.shape(tensor)[mode])), **tl.context(tensor))
-                # factor[:, :tensor.shape[mode]] = U
                 random_part = tl.tensor(rng.random_sample((U.shape[0], rank - tl.shape(tensor)[mode])), **tl.context(tensor))
                 U = tl.concatenate([U, random_part], axis=1)
 
@@ -79,8 +74,20 @@ def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None,
 
     elif isinstance(init, (tuple, list, CPTensor)):
         # TODO: Test this
-        try:
+        try:         
+            if normalize_factors is True:
+                warnings.warn('It is not recommended to initialize a tensor with normalizing. Consider normalizing the tensor before using this function')
+            
             kt = CPTensor(init)
+            weights, factors = kt
+            
+            if tl.all(weights == 1):
+                kt = CPTensor((None, factors))
+            else:
+                weights_avg = tl.prod(weights) ** (1.0 / tl.shape(weights)[0])
+                for i in range(len(factors)):
+                    factors[i] = factors[i] * weights_avg
+                kt = CPTensor((None, factors))
         except ValueError:
             raise ValueError(
                 'If initialization method is a mapping, then it must '
@@ -89,9 +96,6 @@ def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None,
     else:
         raise ValueError('Initialization method "{}" not recognized'.format(init))
 
-    if non_negative:
-        kt.factors = [tl.abs(f) for f in kt[1]]
-
     if normalize_factors:
         kt = cp_normalize(kt)
 
@@ -113,7 +117,7 @@ def sparsify_tensor(tensor, card):
     """
     if card >= np.prod(tensor.shape):
         return tensor
-    bound = tl.sort(tl.abs(tensor), axis = None)[-card]
+    bound = tl.sort(tl.abs(tensor), axis=None)[-card]
     
     return tl.where(tl.abs(tensor) < bound, tl.zeros(tensor.shape, **tl.context(tensor)), tensor)
 
@@ -155,7 +159,7 @@ def error_calc(tensor, norm_tensor, weights, factors, sparsity, mask, mttkrp=Non
 
         # Update the tensor based on the mask
         if mask is not None:
-            tensor = tensor*mask + low_rank_component*(1-mask)
+            tensor = tensor * mask + low_rank_component * (1 - mask)
             norm_tensor = tl.norm(tensor, 2)
 
         if sparsity:
@@ -176,26 +180,26 @@ def error_calc(tensor, norm_tensor, weights, factors, sparsity, mask, mttkrp=Non
 
             # mttkrp and factor for the last mode. This is equivalent to the
             # inner product <tensor, factorization>
-            iprod = tl.sum(tl.sum(mttkrp*factors[-1], axis=0)*weights)
-            unnorml_rec_error = tl.sqrt(tl.abs(norm_tensor**2 + factors_norm**2 - 2*iprod))
+            iprod = tl.sum(tl.sum(mttkrp * factors[-1], axis=0) * weights)
+            unnorml_rec_error = tl.sqrt(tl.abs(norm_tensor**2 + factors_norm**2 - 2 * iprod))
 
     return unnorml_rec_error, tensor, norm_tensor
 
 
-def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
-            normalize_factors=False, orthogonalise=False,\
-            tol=1e-8, random_state=None,\
-            verbose=0, return_errors=False,\
-            sparsity = None,\
-            l2_reg = 0,  mask=None,\
-            cvg_criterion = 'abs_rec_error',\
-            fixed_modes = [],
+def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',
+            normalize_factors=False, orthogonalise=False,
+            tol=1e-8, random_state=None,
+            verbose=0, return_errors=False,
+            sparsity=None,
+            l2_reg=0,  mask=None,
+            cvg_criterion='abs_rec_error',
+            fixed_modes=None,
             svd_mask_repeats=5,
-            linesearch = False):
+            linesearch=False):
     """CANDECOMP/PARAFAC decomposition via alternating least squares (ALS)
-    Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
+    Computes a rank-`rank` decomposition of `tensor` [1]_ such that::
 
-        ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
+        tensor = [|weights; factors[0], ..., factors[-1] |].
 
     Parameters
     ----------
@@ -226,11 +230,11 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
         True). Allows for missing values [2]_
     cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
        Stopping criterion for ALS, works if `tol` is not None. 
-       If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
-       If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
+       If 'rec_error',  ALS stops at current iteration if ``(previous rec_error - current rec_error) < tol``.
+       If 'abs_rec_error', ALS terminates when `|previous rec_error - current rec_error| < tol`.
     sparsity : float or int
         If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
-    fixed_modes : list, default is []
+    fixed_modes : list, default is None
         A list of modes for which the initial value is not modified.
         The last mode cannot be fixed due to error computation.
     svd_mask_repeats: int
@@ -243,10 +247,11 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
     -------
     CPTensor : (weight, factors)
         * weights : 1D array of shape (rank, )
-            all ones if normalize_factors is False (default), 
-            weights of the (normalized) factors otherwise
-        * factors : List of factors of the CP decomposition element `i` is of shape
-            (tensor.shape[i], rank)
+
+          * all ones if normalize_factors is False (default)
+          * weights of the (normalized) factors otherwise
+
+        * factors : List of factors of the CP decomposition element `i` is of shape ``(tensor.shape[i], rank)``
         * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
 
     errors : list
@@ -254,14 +259,14 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
 
     References
     ----------
-    .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
-       SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
+    .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications", SIAM 
+           REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
 
     .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
-            Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
+           Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
 
     .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
-            Applications", PhD., University of Amsterdam, 1998
+           Applications", PhD., University of Amsterdam, 1998
     """
     rank = validate_cp_rank(tl.shape(tensor), rank=rank)
     
@@ -279,13 +284,16 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
 
     if mask is not None and init == "svd":
         for _ in range(svd_mask_repeats):
-            tensor = tensor*mask + tl.cp_to_tensor((weights, factors), mask=1-mask)
+            tensor = tensor * mask + tl.cp_to_tensor((weights, factors), mask=1 - mask)
 
             weights, factors = initialize_cp(tensor, rank, init=init, svd=svd, random_state=random_state, normalize_factors=normalize_factors)
 
     rec_errors = []
     norm_tensor = tl.norm(tensor, 2)
-    Id = tl.eye(rank, **tl.context(tensor))*l2_reg
+    Id = tl.eye(rank, **tl.context(tensor)) * l2_reg
+    
+    if fixed_modes is None:
+        fixed_modes = []
 
     if tl.ndim(tensor)-1 in fixed_modes:
         warnings.warn('You asked for fixing the last mode, which is not supported.\n The last mode will not be fixed. Consider using tl.moveaxis()')
@@ -316,17 +324,13 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
             pseudo_inverse = tl.tensor(np.ones((rank, rank)), **tl.context(tensor))
             for i, factor in enumerate(factors):
                 if i != mode:
-                    pseudo_inverse = pseudo_inverse*tl.dot(tl.conj(tl.transpose(factor)), factor)
+                    pseudo_inverse = pseudo_inverse * tl.dot(tl.transpose(factor), factor)
             pseudo_inverse += Id
 
-            if not iteration and weights is not None:
-                # Take into account init weights
-                mttkrp = unfolding_dot_khatri_rao(tensor, (weights, factors), mode)
-            else:
-                mttkrp = unfolding_dot_khatri_rao(tensor, (None, factors), mode)
+            mttkrp = unfolding_dot_khatri_rao(tensor, (None, factors), mode)
 
-            factor = tl.transpose(tl.solve(tl.conj(tl.transpose(pseudo_inverse)),
-                                    tl.transpose(mttkrp)))
+            factor = tl.transpose(tl.solve(tl.transpose(pseudo_inverse),
+                                  tl.transpose(mttkrp)))
 
             if normalize_factors:
                 scales = tl.norm(factor, 2, axis=0)
@@ -346,14 +350,14 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
             unnorml_rec_error, tensor, norm_tensor = error_calc(tensor, norm_tensor, weights, factors, sparsity, mask, mttkrp)
         else:
             if mask is not None:
-                tensor = tensor*mask + tl.cp_to_tensor((weights, factors), mask=1-mask)
+                tensor = tensor * mask + tl.cp_to_tensor((weights, factors), mask=1 - mask)
 
         # Start line search if requested.
         if line_iter is True:
             jump = iteration ** (1.0 / acc_pow)
 
             new_weights = weights_last + (weights - weights_last) * jump
-            new_factors = [factors_last[ii] + (factors[ii] - factors_last[ii])*jump for ii in range(tl.ndim(tensor))]
+            new_factors = [factors_last[ii] + (factors[ii] - factors_last[ii]) * jump for ii in range(tl.ndim(tensor))]
 
             new_rec_error, new_tensor, new_norm_tensor = error_calc(tensor, norm_tensor, new_weights, new_factors, sparsity, mask)
 
@@ -393,7 +397,7 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
                 if cvg_criterion == 'abs_rec_error':
                     stop_flag = abs(rec_error_decrease) < tol
                 elif cvg_criterion == 'rec_error':
-                    stop_flag =  rec_error_decrease < tol
+                    stop_flag = rec_error_decrease < tol
                 else:
                     raise TypeError("Unknown convergence criterion")
                 
@@ -409,8 +413,7 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
     cp_tensor = CPTensor((weights, factors))
     
     if sparsity:
-        sparse_component = sparsify_tensor(tensor -\
-                                           cp_to_tensor((weights, factors)),\
+        sparse_component = sparsify_tensor(tensor - cp_to_tensor((weights, factors)),
                                            sparsity)
         cp_tensor = (cp_tensor, sparse_component)
 
@@ -420,146 +423,6 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
         return cp_tensor
     
 
-def non_negative_parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',
-                         tol=10e-7, random_state=None, verbose=0, normalize_factors=False,
-                         return_errors=False, mask=None, orthogonalise=False, cvg_criterion='abs_rec_error',
-                         fixed_modes=[]):
-    """
-    Non-negative CP decomposition
-
-    Uses multiplicative updates, see [2]_
-
-    This is the same as parafac(non_negative=True).
-
-    Parameters
-    ----------
-    tensor : ndarray
-    rank   : int
-            number of components
-    n_iter_max : int
-                 maximum number of iteration
-    init : {'svd', 'random'}, optional
-    svd : str, default is 'numpy_svd'
-        function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
-    tol : float, optional
-          tolerance: the algorithm stops when the variation in
-          the reconstruction error is less than the tolerance
-    random_state : {None, int, np.random.RandomState}
-    verbose : int, optional
-        level of verbosity
-    fixed_modes : list, default is []
-        A list of modes for which the initial value is not modified.
-        The last mode cannot be fixed due to error computation.
-
-    Returns
-    -------
-    factors : ndarray list
-            list of positive factors of the CP decomposition
-            element `i` is of shape ``(tensor.shape[i], rank)``
-
-    References
-    ----------
-    .. [2] Amnon Shashua and Tamir Hazan,
-       "Non-negative tensor factorization with applications to statistics and computer vision",
-       In Proceedings of the International Conference on Machine Learning (ICML),
-       pp 792-799, ICML, 2005
-    """
-    epsilon = 10e-12
-    rank = validate_cp_rank(tl.shape(tensor), rank=rank)
-
-    if mask is not None and init == "svd":
-        message = "Masking occurs after initialization. Therefore, random initialization is recommended."
-        warnings.warn(message, Warning)
-
-    if orthogonalise and not isinstance(orthogonalise, int):
-        orthogonalise = n_iter_max
-
-    weights, factors = initialize_cp(tensor, rank, init=init, svd=svd,
-                                 random_state=random_state,
-                                 non_negative=True,
-                                 normalize_factors=normalize_factors)
-    rec_errors = []
-    norm_tensor = tl.norm(tensor, 2)
-
-    if tl.ndim(tensor)-1 in fixed_modes:
-        warnings.warn('You asked for fixing the last mode, which is not supported while tol is fixed.\n The last mode will not be fixed. Consider using tl.moveaxis()')
-        fixed_modes.remove(tl.ndim(tensor)-1)
-    modes_list = [mode for mode in range(tl.ndim(tensor)) if mode not in fixed_modes]
-
-    for iteration in range(n_iter_max):
-        if orthogonalise and iteration <= orthogonalise:
-            for i, f in enumerate(factors):
-                if min(tl.shape(f)) >= rank:
-                    factors[i] = tl.abs(tl.qr(f)[0])
-
-        if verbose > 1:
-            print("Starting iteration", iteration + 1)
-        for mode in modes_list:
-            if verbose > 1:
-                print("Mode", mode, "of", tl.ndim(tensor))
-
-            accum = 1
-            # khatri_rao(factors).tl.dot(khatri_rao(factors))
-            # simplifies to multiplications
-            sub_indices = [i for i in range(len(factors)) if i != mode]
-            for i, e in enumerate(sub_indices):
-                if i:
-                    accum *= tl.dot(tl.transpose(factors[e]), factors[e])
-                else:
-                    accum = tl.dot(tl.transpose(factors[e]), factors[e])
-
-            if mask is not None:
-                tensor = tensor*mask + tl.cp_to_tensor((None, factors), mask=1-mask)
-
-            mttkrp = unfolding_dot_khatri_rao(tensor, (None, factors), mode)
-
-            numerator = tl.clip(mttkrp, a_min=epsilon, a_max=None)
-            denominator = tl.dot(factors[mode], accum)
-            denominator = tl.clip(denominator, a_min=epsilon, a_max=None)
-            factor = factors[mode] * numerator / denominator
-
-            factors[mode] = factor
-
-        if normalize_factors:
-            weights, factors = cp_normalize((weights, factors))
-
-        if tol:
-            # ||tensor - rec||^2 = ||tensor||^2 + ||rec||^2 - 2*<tensor, rec>
-            factors_norm = cp_norm((weights, factors))
-
-            # mttkrp and factor for the last mode. This is equivalent to the
-            # inner product <tensor, factorization>
-            iprod = tl.sum(tl.sum(mttkrp*factor, axis=0)*weights)
-            rec_error = tl.sqrt(tl.abs(norm_tensor**2 + factors_norm**2 - 2*iprod)) / norm_tensor
-            rec_errors.append(rec_error)
-            if iteration >= 1:
-                rec_error_decrease = rec_errors[-2] - rec_errors[-1]
-                
-                if verbose:
-                    print("iteration {}, reconstraction error: {}, decrease = {}".format(iteration, rec_error, rec_error_decrease))
-
-                if cvg_criterion == 'abs_rec_error':
-                    stop_flag = abs(rec_error_decrease) < tol
-                elif cvg_criterion == 'rec_error':
-                    stop_flag =  rec_error_decrease < tol
-                else:
-                    raise TypeError("Unknown convergence criterion")
-                
-                if stop_flag:
-                    if verbose:
-                        print("PARAFAC converged after {} iterations".format(iteration))
-                    break 
-            else:
-                if verbose:
-                    print('reconstruction error={}'.format(rec_errors[-1]))
-
-    cp_tensor = CPTensor((weights, factors))
-
-    if return_errors:
-        return cp_tensor, rec_errors
-    else:
-        return cp_tensor
-
 def sample_khatri_rao(matrices, n_samples, skip_matrix=None,
                       return_sampled_rows=False, random_state=None):
     """Random subsample of the Khatri-Rao product of the given list of matrices
@@ -600,7 +463,7 @@ def sample_khatri_rao(matrices, n_samples, skip_matrix=None,
         list of length `n_samples` containing the sampled row indices
     """
     if random_state is None or not isinstance(random_state, np.random.RandomState):
-        rng = check_random_state(random_state)
+        rng = tl.check_random_state(random_state)
         warnings.warn('You are creating a new random number generator at each call.\n'
                       'If you are calling sample_khatri_rao inside a loop this will be slow:'
                       ' best to create a rng outside and pass it as argument (random_state=rng).')
@@ -619,12 +482,12 @@ def sample_khatri_rao(matrices, n_samples, skip_matrix=None,
         # Compute corresponding rows of the full khatri-rao product
         indices_kr = np.zeros((n_samples), dtype=int)
         for size, indices in zip(sizes, indices_list):
-            indices_kr = indices_kr*size + indices
+            indices_kr = indices_kr * size + indices
 
     # Compute the Khatri-Rao product for the chosen indices
     sampled_kr = tl.ones((n_samples, rank), **tl.context(matrices[0]))
     for indices, matrix in zip(indices_list, matrices):
-        sampled_kr = sampled_kr*matrix[indices, :]
+        sampled_kr = sampled_kr * matrix[indices, :]
 
     if return_sampled_rows:
         return sampled_kr, indices_list, indices_kr
@@ -673,7 +536,7 @@ def randomised_parafac(tensor, rank, n_samples, n_iter_max=100, init='random', s
     """
     rank = validate_cp_rank(tl.shape(tensor), rank=rank)
 
-    rng = check_random_state(random_state)
+    rng = tl.check_random_state(random_state)
     weights, factors = initialize_cp(tensor, rank, init=init, svd=svd, random_state=random_state)
     rec_errors = []
     n_dims = tl.ndim(tensor)
@@ -728,85 +591,87 @@ def randomised_parafac(tensor, rank, n_samples, n_iter_max=100, init='random', s
 class CP(DecompositionMixin):
     """Candecomp-Parafac decomposition via Alternating-Least Square
 
-        Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
+    Computes a rank-`rank` decomposition of `tensor` [1]_ such that::
 
-            ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
+        tensor = [|weights; factors[0], ..., factors[-1] |].
 
-        Parameters
-        ----------
-        tensor : ndarray
-        rank  : int
-            Number of components.
-        n_iter_max : int
-            Maximum number of iteration
-        init : {'svd', 'random'}, optional
-            Type of factor matrix initialization. See `initialize_factors`.
-        svd : str, default is 'numpy_svd'
-            function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
-        normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
-            of shape (rank, ), which will contain the norms of the factors
-        tol : float, optional
-            (Default: 1e-6) Relative reconstruction error tolerance. The
-            algorithm is considered to have found the global minimum when the
-            reconstruction error is less than `tol`.
-        random_state : {None, int, np.random.RandomState}
-        verbose : int, optional
-            Level of verbosity
-        return_errors : bool, optional
-            Activate return of iteration errors
-        mask : ndarray
-            array of booleans with the same shape as ``tensor`` should be 0 where
-            the values are missing and 1 everywhere else. Note:  if tensor is
-            sparse, then mask should also be sparse with a fill value of 1 (or
-            True). Allows for missing values [2]_
-        cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
-            Stopping criterion for ALS, works if `tol` is not None. 
-            If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
-            If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
-        sparsity : float or int
-            If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
-        fixed_modes : list, default is []
-            A list of modes for which the initial value is not modified.
-            The last mode cannot be fixed due to error computation.
-        svd_mask_repeats: int
-            If using a tensor with masked values, this initializes using SVD multiple times to
-            remove the effect of these missing values on the initialization.
-        linesearch : bool, default is False
-            Whether to perform line search as proposed by Bro [3].
+    Parameters
+    ----------
+    tensor : ndarray
+    rank  : int
+        Number of components.
+    n_iter_max : int
+        Maximum number of iteration
+    init : {'svd', 'random'}, optional
+        Type of factor matrix initialization. See `initialize_factors`.
+    svd : str, default is 'numpy_svd'
+        function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
+    normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
+        of shape (rank, ), which will contain the norms of the factors
+    tol : float, optional
+        (Default: 1e-6) Relative reconstruction error tolerance. The
+        algorithm is considered to have found the global minimum when the
+        reconstruction error is less than `tol`.
+    random_state : {None, int, np.random.RandomState}
+    verbose : int, optional
+        Level of verbosity
+    return_errors : bool, optional
+        Activate return of iteration errors
+    mask : ndarray
+        array of booleans with the same shape as ``tensor`` should be 0 where
+        the values are missing and 1 everywhere else. Note:  if tensor is
+        sparse, then mask should also be sparse with a fill value of 1 (or
+        True). Allows for missing values [2]_
+    cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
+        Stopping criterion for ALS, works if `tol` is not None. 
+        If 'rec_error',  ALS stops at current iteration if ``(previous rec_error - current rec_error) < tol``.
+        If 'abs_rec_error', ALS terminates when `|previous rec_error - current rec_error| < tol`.
+    sparsity : float or int
+        If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
+    fixed_modes : list, default is None
+        A list of modes for which the initial value is not modified.
+        The last mode cannot be fixed due to error computation.
+    svd_mask_repeats: int
+        If using a tensor with masked values, this initializes using SVD multiple times to
+        remove the effect of these missing values on the initialization.
+    linesearch : bool, default is False
+        Whether to perform line search as proposed by Bro [3].
 
-        Returns
-        -------
-        CPTensor : (weight, factors)
-            * weights : 1D array of shape (rank, )
-                all ones if normalize_factors is False (default), 
-                weights of the (normalized) factors otherwise
-            * factors : List of factors of the CP decomposition element `i` is of shape
-                (tensor.shape[i], rank)
-            * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
+    Returns
+    -------
+    CPTensor : (weight, factors)
+        * weights : 1D array of shape (rank, )
+          
+          * all ones if normalize_factors is False (default), 
+          * weights of the (normalized) factors otherwise
 
-        errors : list
-            A list of reconstruction errors at each iteration of the algorithms.
+        * factors : List of factors of the CP decomposition element `i ` is of shape
+          (tensor.shape[i], rank)
+        * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
 
-        References
-        ----------
-        .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
-        SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
+    errors : list
+        A list of reconstruction errors at each iteration of the algorithms.
 
-        .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
-                Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
+    References
+    ----------
+    .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
+           SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
 
-        .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
-                Applications", PhD., University of Amsterdam, 1998
+    .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
+           Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
+
+    .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
+           Applications", PhD., University of Amsterdam, 1998
     """
     def __init__(self, rank, n_iter_max=100, tol=1e-08, 
                  init='svd', svd='numpy_svd',
                  l2_reg=0,
                  linesearch=False,
-                 fixed_modes = [],
+                 fixed_modes=None,
                  normalize_factors=False, 
                  orthogonalise=False, 
-                 sparsity = None,
-                 mask=None, svd_mask_repeats = 5,
+                 sparsity=None,
+                 mask=None, svd_mask_repeats=5,
                  cvg_criterion='abs_rec_error',
                  random_state=None, 
                  verbose=0):
@@ -840,18 +705,18 @@ class CP(DecompositionMixin):
             decomposed tensor
         """
         cp_tensor, errors = parafac(tensor, rank=self.rank,
-                        n_iter_max=self.n_iter_max,
-                        tol=self.tol,
-                        init=self.init,
-                        svd=self.svd,
-                        normalize_factors=self.normalize_factors,
-                        orthogonalise=self.orthogonalise,
-                        mask=self.mask,
-                        linesearch = self.linesearch,
-                        cvg_criterion=self.cvg_criterion,
-                        random_state=self.random_state,
-                        verbose=self.verbose,
-                        return_errors=True)
+                                    n_iter_max=self.n_iter_max,
+                                    tol=self.tol,
+                                    init=self.init,
+                                    svd=self.svd,
+                                    normalize_factors=self.normalize_factors,
+                                    orthogonalise=self.orthogonalise,
+                                    mask=self.mask,
+                                    linesearch=self.linesearch,
+                                    cvg_criterion=self.cvg_criterion,
+                                    random_state=self.random_state,
+                                    verbose=self.verbose,
+                                    return_errors=True)
         self.decomposition_ = cp_tensor 
         self.errors_ = errors
         return self.decomposition_
@@ -860,141 +725,6 @@ class CP(DecompositionMixin):
         return f'Rank-{self.rank} CP decomposition.'
 
 
-
-class CPNN(DecompositionMixin):
-    """Non-Negative Candecomp-Parafac decomposition via Alternating-Least Square
-
-        Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
-
-            ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
-
-        Parameters
-        ----------
-        tensor : ndarray
-        rank  : int
-            Number of components.
-        n_iter_max : int
-            Maximum number of iteration
-        init : {'svd', 'random'}, optional
-            Type of factor matrix initialization. See `initialize_factors`.
-        svd : str, default is 'numpy_svd'
-            function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
-        normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
-            of shape (rank, ), which will contain the norms of the factors
-        tol : float, optional
-            (Default: 1e-6) Relative reconstruction error tolerance. The
-            algorithm is considered to have found the global minimum when the
-            reconstruction error is less than `tol`.
-        random_state : {None, int, np.random.RandomState}
-        verbose : int, optional
-            Level of verbosity
-        return_errors : bool, optional
-            Activate return of iteration errors
-        mask : ndarray
-            array of booleans with the same shape as ``tensor`` should be 0 where
-            the values are missing and 1 everywhere else. Note:  if tensor is
-            sparse, then mask should also be sparse with a fill value of 1 (or
-            True). Allows for missing values [2]_
-        cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
-            Stopping criterion for ALS, works if `tol` is not None. 
-            If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
-            If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
-        sparsity : float or int
-            If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
-        fixed_modes : list, default is []
-            A list of modes for which the initial value is not modified.
-            The last mode cannot be fixed due to error computation.
-        svd_mask_repeats: int
-            If using a tensor with masked values, this initializes using SVD multiple times to
-            remove the effect of these missing values on the initialization.
-        linesearch : bool, default is False
-            Whether to perform line search as proposed by Bro [3].
-
-        Returns
-        -------
-        CPTensor : (weight, factors)
-            * weights : 1D array of shape (rank, )
-                all ones if normalize_factors is False (default), 
-                weights of the (normalized) factors otherwise
-            * factors : List of factors of the CP decomposition element `i` is of shape
-                (tensor.shape[i], rank)
-            * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
-
-        errors : list
-            A list of reconstruction errors at each iteration of the algorithms.
-
-        References
-        ----------
-        .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
-        SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
-
-        .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
-                Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
-
-        .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
-                Applications", PhD., University of Amsterdam, 1998
-    """
-    def __init__(self, rank, n_iter_max=100, tol=1e-08, 
-                 init='svd', svd='numpy_svd',
-                 l2_reg=0,
-                 linesearch=False,
-                 fixed_modes = [],
-                 normalize_factors=False, 
-                 orthogonalise=False, 
-                 sparsity = None,
-                 mask=None, svd_mask_repeats = 5,
-                 cvg_criterion='abs_rec_error',
-                 random_state=None, 
-                 verbose=0):
-        self.rank = rank
-        self.n_iter_max = n_iter_max
-        self.tol = tol
-        self.l2_reg = l2_reg
-        self.init = init
-        self.linesearch = linesearch
-        self.svd = svd
-        self.normalize_factors = normalize_factors
-        self.orthogonalise = orthogonalise
-        self.mask = mask
-        self.svd_mask_repeats = svd_mask_repeats
-        self.cvg_criterion = cvg_criterion
-        self.random_state = random_state
-        self.verbose = verbose
-
-    
-    def fit_transform(self, tensor):
-        """Decompose an input tensor
-
-        Parameters
-        ----------
-        tensor : tensorly tensor
-            input tensor to decompose
-
-        Returns
-        -------
-        CPTensor
-            decomposed tensor
-        """
-        cp_tensor, errors = non_negative_parafac(tensor, rank=self.rank,
-                        n_iter_max=self.n_iter_max,
-                        tol=self.tol,
-                        init=self.init,
-                        svd=self.svd,
-                        normalize_factors=self.normalize_factors,
-                        orthogonalise=self.orthogonalise,
-                        mask=self.mask,
-                        cvg_criterion=self.cvg_criterion,
-                        random_state=self.random_state,
-                        verbose=self.verbose,
-                        return_errors=True)
-        self.decomposition_ = cp_tensor 
-        self.errors_ = errors
-        return self.decomposition_
-
-    def __repr__(self):
-        return f'Rank-{self.rank} Non-Negative CP decomposition.'
-
-
 class RandomizedCP(DecompositionMixin):
     """Randomised CP decomposition via sampled ALS
 
@@ -1029,19 +759,19 @@ class RandomizedCP(DecompositionMixin):
     References
     ----------
     .. [3] Casey Battaglino, Grey Ballard and Tamara G. Kolda,
-    "A Practical Randomized CP Tensor Decomposition",
+       "A Practical Randomized CP Tensor Decomposition",
     """
     def __init__(self, rank, n_samples, n_iter_max=100, init='random', svd='numpy_svd',
-                       tol=10e-9, max_stagnation=20, random_state=None, verbose=1):
-        self.rank=rank
-        self.n_samples=n_samples
-        self.n_iter_max=n_iter_max
-        self.init=init
-        self.svd=svd
-        self.tol=tol
-        self.max_stagnation=max_stagnation
-        self.random_state=random_state
-        self.verbose=verbose
+                 tol=10e-9, max_stagnation=20, random_state=None, verbose=1):
+        self.rank = rank
+        self.n_samples = n_samples
+        self.n_iter_max = n_iter_max
+        self.init = init
+        self.svd = svd
+        self.tol = tol
+        self.max_stagnation = max_stagnation
+        self.random_state = random_state
+        self.verbose = verbose
 
     def fit_transform(self, tensor):
         self.decomposition_, self.errors_ = randomised_parafac(tensor, rank=self.rank, n_samples=self.n_samples,
