@@ -91,19 +91,23 @@ def tensor_train_matrix(tensor, rank):
         msg = 'The tensor should have as many dimensions for inputs and outputs, i.e. order should be even '
         msg += f'but got a tensor of order tl.ndim(tensor)={order} which is odd.'
         raise ValueError(msg)
-        
+
     in_shape = tl.shape(tensor)[:n_input]
     out_shape = tl.shape(tensor)[n_input:]
 
+    if n_input == 1:
+        # A TTM with a single factor is just a matrix...
+        return TTMatrix([tensor.reshape(1, in_shape[0], out_shape[0], 1)])
+
     new_idx = list([idx for tuple_ in zip(range(n_input), range(n_input, 2*n_input)) for idx in tuple_])
     new_shape = list([a*b for (a,b) in zip(in_shape, out_shape)])
     tensor = tl.reshape(tl.transpose(tensor, new_idx), new_shape)
     
-    factors = tensor_train(tensor, rank)
+    factors = tensor_train(tensor, rank).factors
     for i in range(len(factors)):
         factors[i] = tl.reshape(factors[i], (factors[i].shape[0], in_shape[i], out_shape[i], -1))
     
-    return factors
+    return TTMatrix(factors)
 
 
 class TensorTrain(DecompositionMixin):
@@ -139,4 +143,4 @@ class TensorTrain(DecompositionMixin):
         self.decomposition_ = tensor_train(tensor, rank=self.rank, verbose=self.verbose)
         return self.decomposition_
 
-matrix_product_state = DefineDeprecated('matrix_product_state', tensor_train)
\ No newline at end of file
+matrix_product_state = DefineDeprecated('matrix_product_state', tensor_train)
