@@ -1,4 +1,4 @@
-from .base import check_random_state, random_tensor
+from .base import random_tensor
 from .base import (random_cp, random_tucker, 
                    random_tt, random_tt_matrix, random_parafac2)
 
