@@ -4,7 +4,7 @@ from ... import backend as tl
 
 # License: BSD 3 clause
 
-def outer(vectors):
+def outer(vectors, weights=None):
     """Returns the outer product of vectors
 
     Parameters
@@ -16,6 +16,9 @@ def outer(vectors):
     -------
     tensor of order len(vectors) with tensor.shape[i] == len(vectors[i])
     """
+    if weights is not None:
+        vec, *vectors = vectors
+        vectors = [vec*weights, *vectors]
     start = ord('a')
     symbols = [chr(start + i) for i in range(len(vectors))]
     equation = ','.join(symbols) + '->' + ''.join(symbols)
