@@ -9,8 +9,8 @@ from ...testing import assert_equal, assert_array_almost_equal, assert_raises
 
 
 def test_inner():
-    tensor_1 = T.tensor(np.arange(3 * 4).reshape((3, 4)))
-    tensor_2 = T.tensor(np.arange(4 * 2).reshape((4, 2)))
+    tensor_1 = T.tensor(np.arange(3 * 4, dtype=float).reshape((3, 4)))
+    tensor_2 = T.tensor(np.arange(4 * 2, dtype=float).reshape((4, 2)))
 
     # For one common mode, equivalent to dot product
     res = inner(tensor_1, tensor_2, n_modes=1)
