@@ -5,14 +5,14 @@ from ..decomposition import tensor_train
 from ..tt_tensor import tt_to_tensor, _validate_tt_tensor
 from ..tt_tensor import validate_tt_rank, _tt_n_param
 from ..testing import assert_array_almost_equal, assert_equal, assert_raises, assert_
-from ..random import check_random_state, random_tt
+from ..random import random_tt
 
 
 def test_validate_tt_tensor():
-    rng = check_random_state(12345)
+    rng = tl.check_random_state(12345)
     true_shape = (3, 4, 5)
     true_rank = (1, 3, 2, 1)
-    factors = random_tt(true_shape, rank=true_rank)
+    factors = random_tt(true_shape, rank=true_rank).factors
     
     # Check that the correct shape/rank are returned
     shape, rank = _validate_tt_tensor(factors)
@@ -39,10 +39,6 @@ def test_validate_tt_tensor():
     with assert_raises(ValueError):
         _validate_tt_tensor(factors)
 
-    # Not enough factors
-    with assert_raises(ValueError):
-        _validate_tt_tensor(factors[:1])
-
 
 def test_tt_to_tensor():
     """ Test for tt_to_tensor
@@ -144,3 +140,4 @@ def testvalidate_tt_rank():
     rank = validate_tt_rank(tensor_shape, rank='same', rounding='ceil')
     n_param = _tt_n_param(tensor_shape, rank)
     assert_(n_param >= n_param_tensor)
+
